#include <bitset>
#include <cassert>
#include <iostream>
#include <stdexcept>
#include <vector>

constexpr long double operator""_km(long double km) {
	return km * 1000.0;
}

constexpr long double operator""_miles(long double miles) {
	return miles * 1609.344;
}

std::vector<bool> operator""_bv(const char* bit_string) {
	std::vector<bool> result;
	while (*bit_string) {
		assert(*bit_string == '0' || *bit_string == '1');
		result.push_back(*bit_string == '1');
		++bit_string;
	}
	return result;
}

template<char ... Bits>
constexpr auto operator""_bs() {
	const size_t Size = sizeof...(Bits);
	char bits[] = { Bits... };

	size_t result = 0;

	for (size_t i = Size; i --> 0; ) {
		if (bits[i] != '0' && bits[i] != '1') {
			throw std::invalid_argument("Bad bitset spec");
		}
		result = (result << 1) | (bits[i] == '1');
	}

	return std::bitset<Size>(result);
}

int main() {
	std::cout << 100.0_km + 200.0_miles << '\n';
	auto bv = 0010011_bv;
	for (bool b : bv) {
		std::cout << b;
	}
	std::cout << '\n';

	// the bitset is created at compile time
	constexpr auto bs = 0010011_bs;
	for (size_t i = 0; i < bs.size(); ++i) {
		std::cout << bs[i];
	}
	std::cout << '\n';
}
